use std::path::PathBuf;

use git2::{Cred, RemoteCallbacks};

use crate::storage::DynStorageEngine;

use super::GitProvider;

pub struct GitHubGitProvider {
    storage_engine: DynStorageEngine,
}

impl GitHubGitProvider {
    pub fn new(storage_engine: DynStorageEngine) -> Self {
        Self { storage_engine }
    }
}

#[async_trait::async_trait]
impl GitProvider for GitHubGitProvider {
    async fn clone_from_url(&self, url: String) -> eyre::Result<PathBuf> {
        tracing::debug!(url, "allocating dir");
        let dir = self.storage_engine.allocate_dir().await?;

        let dirpath = dir.clone().path();
        let _ = tokio::task::spawn_blocking(move || {
            let mut callbacks = RemoteCallbacks::new();
            callbacks.credentials(|url, username_from_url, _allowed_types| {
                tracing::debug!(username_from_url, url, "pulling key from ssh-agent");
                Cred::ssh_key_from_agent(username_from_url.unwrap())
            });

            let mut fo = git2::FetchOptions::new();
            fo.remote_callbacks(callbacks);

            let mut builder = git2::build::RepoBuilder::new();
            builder.fetch_options(fo);

            tracing::debug!(
                url,
                path = dirpath.as_os_str().to_string_lossy().to_string(),
                "clone git repo"
            );
            builder.clone(url.as_str(), dirpath.as_path())
        })
        .await??;

        tracing::debug!("done pulling repo");

        Ok(dir.path())
    }
}
